import 'package:osam/osam.dart';
import 'package:osam_flutter/src/navigation/osam_route.dart';

import '../../osam_flutter.dart';

mixin Navigation {
  PropertyNotifier<List<OsamRoute<Object, Object>>> get routes;

  Future<R> push<A, R>(OsamRoute<A, R> route) {
    routes.update((value) => value.add(route));
    return route.whenComplete;
  }

  void pop() => routes.update((value) => value.removeLast());

  OsamRoute<Object, Object> findRoute(String route) =>
      routes.value.firstWhere((element) => element.value == route, orElse: throw NoRouteException());

  Future<R> pushReplacement<A, R>(OsamRoute<A, R> route) {
    routes.update((value) {
      value.removeLast();
      value.add(route);
    });
    return route.whenComplete;
  }

  Future<R> pushAndRemoveUntil<A, R>(OsamRoute<A, R> route) {
    routes.update((value) {
      value.clear();
      value.add(route);
    });
    return route.whenComplete;
  }

  void popUntil(String route) => routes.update((value) {
        if (value.contains(route)) {
          while (value.last.value != route) {
            value.removeLast();
          }
        }
      });
}

mixin TabNavigation {
  PropertyNotifier<String> get tabNotifier;

  set tab(String tab) => tabNotifier.value = tab;
}
